﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Threading;
using System.IO.Ports;

namespace WpfApp1
{
    /// <summary>
    /// MainWindow.xaml에 대한 상호 작용 논리
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Initialized(object sender, EventArgs e)
        {
            renderer = new GraphRenderer[] {
                new GraphRenderer(canvas1, Colors.Red),
                new GraphRenderer(canvas2, Colors.Orange),
                new GraphRenderer(canvas3, Colors.DarkKhaki),
                new GraphRenderer(canvas4, Colors.CornflowerBlue),
                new GraphRenderer(canvas5, Colors.Green),
                new GraphRenderer(canvas6, Colors.Blue),
                new GraphRenderer(canvas7, Colors.Purple),
                new GraphRenderer(canvas8, Colors.Chocolate),
            };

            _thrd = new Thread(InputThread);
            _thrd.Start();
        }

        private void Window_Closed(object sender, EventArgs e)
        {
            _thrd.Abort();
            _thrd.Join();
        }

        private void InputThread()
        {
            SerialPort port = new SerialPort("COM8", 115200);
            try
            {
                port.Open();

                while (true)
                {
                    try
                    {
                        string input = port.ReadLine();
                        if (input.FirstOrDefault() != '=')
                            continue;

                        double[] values = input.Substring(1).Split(" \t\r\n".ToArray()).Where(x => x.Length > 0)
                            .Select(x => (double)uint2float(Convert.ToUInt32(x, 16))).ToArray();
                        if (values.Length != 6)
                            continue;

                        Dispatcher.InvokeAsync(() =>
                        {
                            renderer[0].AddValue(values[0]);
                            renderer[1].AddValue(values[1]);
                            renderer[2].AddValue(values[2]);
                            renderer[4].AddValue(values[3]);
                            renderer[5].AddValue(values[4]);
                            renderer[6].AddValue(values[5]);

                            double acc = Math.Sqrt(values[0] * values[0] + values[1] * values[1] + values[2] * values[2]);
                            double gyro = Math.Sqrt(values[3] * values[3] + values[4] * values[4] + values[5] * values[5]);
                            renderer[3].AddValue(acc);
                            renderer[7].AddValue(gyro);
                        });
                    }
                    catch (FormatException) { }
                    catch (TimeoutException) { }
                    catch (OverflowException) { }
                }
            }
            catch (InvalidOperationException)
            {
                Dispatcher.Invoke(() =>
                {
                    MessageBox.Show("arduino connection closed");
                    Close();
                });
            }
            finally
            {
                port.Close();
            }
        }

        private static unsafe float uint2float(uint value)
        {
            return *(float*)(&value);
        }

        private Thread _thrd;

        private GraphRenderer[] renderer;
    }

    class GraphRenderer
    {
        public Canvas Canvas { get; }
        public Color Color { get; set; }

        public double Time { get; set; }
        public double DeltaTime { get; set; }
        public double Value { get; set; }

        private TranslateTransform translate = new TranslateTransform();
        private ScaleTransform scale = new ScaleTransform();

        private List<Line> lines = new List<Line>();

        public GraphRenderer(Canvas canvas, Color color)
        {
            Canvas = canvas;
            Color = color;
            Time = 0;
            Value = 0;

            DeltaTime = 1;

            var transform = new TransformGroup();
            transform.Children.Add(translate);
            transform.Children.Add(scale);
            Canvas.RenderTransform = transform;
        }

        public void AddValue(double newValue)
        {
            Line line = new Line();
            line.X1 = Time;
            line.Y1 = 10 * Value;
            line.X2 = Time + DeltaTime;
            line.Y2 = 10 * newValue;
            line.StrokeThickness = 1;
            line.Stroke = new SolidColorBrush(Color);
            Canvas.Children.Add(line);
            translate.X = -Time;

            lines.Add(line);
            if (lines.Count > 1000)
            {
                Canvas.Children.Remove(lines[0]);
                lines.RemoveAt(0);
            }

            Value = newValue;
            Time += DeltaTime;
        }
    }
}
